%{
import YYtokentypes;
%}

%x COMMENT

IDCHAR [A-Za-z_$]
DIGIT [0-9]
HEXDIGIT [0-9a-fA-F]
OCTDIGIT [0-7]
NONL_WHITESPACE [\ \t\b\f\r]
WHITESPACE [\n\ \t\b\r\f]
UNICODE (\\u{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}{HEXDIGIT}{HEXDIGIT})
OCTCODE (\\{OCTDIGIT}|\\{OCTDIGIT}{OCTDIGIT}|\\{OCTDIGIT}{OCTDIGIT}{OCTDIGIT})
STRING_TEXT ({UNICODE}|{OCTCODE}|\\\"|[^\n\"]|\\{WHITESPACE}+\\)*
SQUOTE_TEXT ({UNICODE}|{OCTCODE}|[^']|\\.)

%% 

<INITIAL>{
"+"	{yyreturn('+');}
"-"	{yyreturn('-');}
"*"	{yyreturn('*');}
"/"	{yyreturn('/');}
"%"	{yyreturn('%');}
"("	{yyreturn('(');}
")"	{yyreturn(')');}
";"	{yyreturn(';');}
"quit"	{yytokentype = YYtokentypes.QUIT;}
"print"	{yytokentype = YYtokentypes.PRINT;}

0[0-7]+		{/* octal */
	yytokentype = YYtokentypes.INTEGER_LITERAL; 
	}

0[xX][0-9a-fA-F]+	{ /* hex */
	yytokentype = YYtokentypes.INTEGER_LITERAL;
	}

{DIGIT}+	{ /* decimal */
	yytokentype = YYtokentypes.INTEGER_LITERAL;
	}

{DIGIT}+\.{DIGIT}*([eE][\+\-]?{DIGIT}+)? { /* float 0.e0 */
	yytokentype = YYtokentypes.FLOAT_LITERAL;
	}

\.{DIGIT}+([eE][\+\-]?{DIGIT}+)? { /* float .0e0 */
	yytokentype = YYtokentypes.FLOAT_LITERAL;
	}

{DIGIT}+[eE][\+\-]?{DIGIT}+	{ /* float 0e-0 */
	yytokentype = YYtokentypes.FLOAT_LITERAL;   
	}

'{SQUOTE_TEXT}'	{ /* quoted char */
		yytokentype = YYtokentypes.CHARACTER_LITERAL;
		}

{NONL_WHITESPACE}+	{}

"//"[^\n]*	{ /*   uni-line comment */ }
"/*"		{ BEGIN(COMMENT); comment_depth++;  }

\n		{}

. { yylexerror("Illegal character: ");}

}

<COMMENT>{
[^*/]+ {}
\*[^/] {}
\/[^*] {}
"/*"	{ /* begin nested multiline comment */ comment_depth++; }
"*/"	{ /* end multiline comment */
		comment_depth--;
		if (comment_depth == 0) { BEGIN(INITIAL); }
		}
. { yylexerror("Illegal character: ");}

}
%%

protected static final char LBRACE = '{';
protected static final char RBRACE = '}';
protected int comment_depth = 0;
