/* Infix notation calculator--calc */
%{
/* put prefix code here; appears before class YYparse */
%}
/* BISON Declarations */
/* list of possible semantic types; default is implied Integer */
%union {YYterminal}

%token <Integer> INTEGER_LITERAL LONG_LITERAL
%token <Integer> FLOAT_LITERAL DOUBLE_LITERAL
%token <Integer> BOOLEAN_LITERAL
%token <Integer> STRING_LITERAL CHARACTER_LITERAL
%token <Integer> IDENTIFIER
%token SHIFTLEFT SHIFTRIGHT LSHIFTRIGHT
%token PRINT QUIT
%token '\'' '"' ','
%left '&' '|'
%left NOT     /* unary not */
%left '-' '+'
%left '*' '/'
%left NEG     /* negation--unary minus */
%right '^'    /* exponentiation        */

%type <Integer> number character
%type <YYterminal> exp

/* Grammar follows */
%%
input:	  cmd opteols
	| input ';' opteols cmd
	;

opteols:  /*null*/
	| opteols '\n'
	;

cmd:	  /*null*/
	| QUIT  {yyreturn(YYACCEPT);}
	| PRINT exp 
	    { System.out.println($2.double_value()); }
	;

exp:
	  number
	    {
		t = new YYterminal($1,yytext);
		d = t.double_value();
		t.settext(Double.toString(d));
		$$ = t;
	    }
	| character
	    {
		t = new YYterminal($1,yytext);
		t.stripquotes();
		// yyout.println("stripped:"+t.length()+":/"+t+"/");
		t.deescape();
		// yyout.println("escaped:"+t.length()+":/"+t+"/");
		if(t.length() != 1) {
		    yyerror("bad single quoted char" + "/" + t + "/");
		}
		d = (double)(int)t.char_value();
		$$ = new YYterminal(DOUBLE_LITERAL,Double.toString(d));
	    }
	| exp '+' exp
	    { $$ = $1; $$.setvalue($1.double_value() + $3.double_value()); }
	| exp '-' exp
	    { $$ = $1; $$.setvalue($1.double_value() - $3.double_value()); }
	| exp '*' exp
	    { $$ = $1; $$.setvalue($1.double_value() * $3.double_value()); }
	| exp '/' exp
	    { $$ = $1; $$.setvalue($1.double_value() / $3.double_value()); }
	| '-' exp  %prec NEG
	    { $$ = $2; $$.setvalue(- $2.double_value()); }
	| '(' exp ')'
	    {$$ = $2;}
	;

number: INTEGER_LITERAL | LONG_LITERAL | DOUBLE_LITERAL | FLOAT_LITERAL  ;

character: CHARACTER_LITERAL ;

%%
/* put suffix code here; appears within class YYlex but at the end */

YYterminal t;
double d;

/* Override yyerror */
public
void
yyverror(String s) throws ParseException
{
    String msg = yyerror_verbose(s);
    System.out.print("yyverror: line ");
    System.out.print(yyloc.lineno()+1);
    System.out.println(" ; " + msg);
    throw new ParseException(s);
}
