%{
import YYtokentypes;
%}

%x COMMENT

IDCHAR [A-Za-z_$]
DIGIT [0-9]
HEXDIGIT [0-9a-fA-F]
OCTDIGIT [0-7]
NONL_WHITESPACE [\ \t\b\f\r]
WHITESPACE [\n\ \t\b\r\f]
UNICODE (\\u{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}{HEXDIGIT}|\\u{HEXDIGIT}{HEXDIGIT}{HEXDIGIT}{HEXDIGIT})
OCTCODE (\\{OCTDIGIT}|\\{OCTDIGIT}{OCTDIGIT}|\\{OCTDIGIT}{OCTDIGIT}{OCTDIGIT})
STRING_TEXT (\\\"|{UNICODE}|{OCTCODE}|[^\n\"]|\\{WHITESPACE}+\\)*
SQUOTE_TEXT ({UNICODE}|{OCTCODE}|[^']|\\.)

%%

<INITIAL>{

"!"		{yyreturn('!');}
"%"		{yyreturn('%');}
"&"		{yyreturn('&');}
"("		{yyreturn('(');}
")"		{yyreturn(')');}
"*"		{yyreturn('*');}
"+"		{yyreturn('+');}
","		{yyreturn(',');}
"-"		{yyreturn('-');}
"."		{yyreturn('.');}
"/"		{yyreturn('/');}
":"		{yyreturn(':');}
";"		{yyreturn(';');}
"<"		{yyreturn('<');}
">"		{yyreturn('>');}
"="		{yyreturn('=');}
"?"		{yyreturn('?');}
"["		{yyreturn('[');}
"]"		{yyreturn(']');}
"^"		{yyreturn('^');}
"{"		{yyreturn(LBRACE);}
"}"		{yyreturn(RBRACE);}
"|"		{yyreturn('|');}
"~"		{yyreturn('~');}
"&&"		{yyreturn(ANDAND);}
"||"		{yyreturn(OROR);}
"+="		{yyreturn(EQADD);}
"&="		{yyreturn(EQAND);}
"/="		{yyreturn(EQDIV);}
"=="		{yyreturn(EQEQ);}
">>>="		{yyreturn(EQLOGRSHIFT);}
"<<="		{yyreturn(EQLSHIFT);}
"%="		{yyreturn(EQMOD);}
"*="		{yyreturn(EQMUL);}
"|="		{yyreturn(EQOR);}
">>="		{yyreturn(EQRSHIFT);}
"-="		{yyreturn(EQSUB);}
"^="		{yyreturn(EQXOR);}
">="		{yyreturn(GE);}
"<="		{yyreturn(LE);}
">>>"		{yyreturn(LOGRSHIFT);}
"<<"		{yyreturn(LSHIFT);}
"--"		{yyreturn(MINUSMINUS);}
"!="		{yyreturn(NE);}
"++"		{yyreturn(PLUSPLUS);}
">>"		{yyreturn(RSHIFT);}

"abstract"	{yyreturn(ABSTRACT);}
"boolean"	{yyreturn(BOOLEAN);}
"break"		{yyreturn(BREAK);}
"byte"		{yyreturn(BYTE);}
"case"		{yyreturn(CASE);}
"catch"		{yyreturn(CATCH);}
"char"		{yyreturn(CHAR);}
"class"		{yyreturn(CLASS);}
"continue"	{yyreturn(CONTINUE);}
"default"	{yyreturn(DEFAULT);}
"do"		{yyreturn(DO);}
"double"	{yyreturn(DOUBLE);}
"else"		{yyreturn(ELSE);}
"extends"	{yyreturn(EXTENDS);}
"final"		{yyreturn(FINAL);}
"finally"	{yyreturn(FINALLY);}
"float"		{yyreturn(FLOAT);}
"for"		{yyreturn(FOR);}
"if"		{yyreturn(IF);}
"implements"	{yyreturn(IMPLEMENTS);}
"import"	{yyreturn(IMPORT);}
"instanceof"	{yyreturn(INSTANCEOF);}
"int"		{yyreturn(INT);}
"interface"	{yyreturn(INTERFACE);}
"long"		{yyreturn(LONG);}
"native"	{yyreturn(NATIVE);}
"new"		{yyreturn(NEW);}
"null"		{yyreturn(NULL);}
"package"	{yyreturn(PACKAGE);}
"private"	{yyreturn(PRIVATE);}
"protected"	{yyreturn(PROTECTED);}
"public"	{yyreturn(PUBLIC);}
"return"	{yyreturn(RETURN);}
"short"		{yyreturn(SHORT);}
"static"	{yyreturn(STATIC);}
"super"		{yyreturn(SUPER);}
"switch"	{yyreturn(SWITCH);}
"synchronized"	{yyreturn(SYNCHRONIZED);}
"this"		{yyreturn(THIS);}
"throw"		{yyreturn(THROW);}
"throws"	{yyreturn(THROWS);}
"transient"	{yyreturn(TRANSIENT);}
"try"		{yyreturn(TRY);}
"void"		{yyreturn(VOID);}
"volatile"	{yyreturn(VOLATILE);}
"while"		{yyreturn(WHILE);}

"true"		{yyreturn(BOOLEAN_LITERAL);}
"false"		{yyreturn(BOOLEAN_LITERAL);}

{IDCHAR}({IDCHAR}|{DIGIT})* { /* identifier */
		yyreturn(IDENTIFIER);
		}

0[0-7]+		{ /*octal*/
		yyreturn(INTEGER_LITERAL);
		}

0[xX][0-9a-fA-F]+	{ /*hex*/
		yyreturn(INTEGER_LITERAL);
		}

{DIGIT}+	{ /*decimal*/
		yyreturn(INTEGER_LITERAL);
		}

{DIGIT}+\.{DIGIT}*([eE][\+\-]?{DIGIT}+)? { /*  float 0.e0 */
		yyreturn(FLOAT_LITERAL);
		}

\.{DIGIT}+([eE][\+\-]?{DIGIT}+)? { /* float .0e0 */
		yyreturn(FLOAT_LITERAL);
		}

{DIGIT}+[eE][\+\-]?{DIGIT}+	{ /* float 0e-0 */
		yyreturn(FLOAT_LITERAL);
		}

'{SQUOTE_TEXT}'	{ /* quoted char */
		yyreturn(YYtokentypes.CHARACTER_LITERAL);
		}


\"{STRING_TEXT}\" { /* string */
		yyreturn(STRING_LITERAL);
		}
\"{STRING_TEXT} { yylexerror("unclosed string");}

"//"[^\n]*	{ /* uni-line comment */ }
"/*"		{ yybegin(COMMENT); comment_depth++; }

{NONL_WHITESPACE}+	{ /* eliminate white space */ }

. { yylexerror("Illegal character: "+yytext); }

}

<COMMENT>{
"/*"	{ /* begin nested multiline comment */ comment_depth++; }
"*/"	{ /* end multiline comment */
		if(--comment_depth == 0) { yybegin(INITIAL); }
		}
[^*/\n]+ {}
\*[^/] {}
\/[^*] {}

.	{yylexerror("Illegal character in comment: " + ((int)yytext.currentchar()));}
}

<*>"\n"	{}

%%

// Include the set of tokentypes to 
// avoid having to say e.g. Tokentypes.INTEGER_LITERAL

// TOKENTYPES

@TOKENTYPES@

protected static final char LBRACE = '{';
protected static final char RBRACE = '}';
protected int comment_depth = 0;
