%expect 1
%token INTEGER_LITERAL LONG_LITERAL
%token FLOAT_LITERAL DOUBLE_LITERAL
%token BOOLEAN_LITERAL
%token STRING_LITERAL CHARACTER_LITERAL
%token IDENTIFIER
%token '!' '%' '&' '(' ')' '*' '+' ',' '-' '.' '/' ':'
%token ';' '<' '>' '=' '?' '[' ']' '^' '{' '}' '|' '~'
%token ABSTRACT ANDAND BOOLEAN BREAK BYTE CASE CATCH CHAR CLASS
%token CONTINUE DEFAULT DO DOUBLE ELSE EQADD EQAND EQDIV
%token EQEQ EQLOGRSHIFT EQLSHIFT EQMOD EQMUL EQOR EQRSHIFT EQSUB
%token EQXOR EXTENDS FINAL FINALLY FLOAT FOR GE IF
%token IMPLEMENTS IMPORT INSTANCEOF INT INTERFACE LE LOGRSHIFT LONG LSHIFT
%token MINUSMINUS NATIVE NE NEW NULL OROR PACKAGE PLUSPLUS PRIVATE PROTECTED
%token PUBLIC RETURN RSHIFT SHORT STATIC SUPER SWITCH SYNCHRONIZED
%token THIS THROW THROWS TRANSIENT TRY VOID VOLATILE WHILE


%right '=' EQMUL EQDIV EQMOD EQADD EQSUB EQLSHIFT EQRSHIFT EQLOGRSHIFT EQAND EQXOR EQOR
%left OROR
%left ANDAND
%left '|'
%left '^'
%left '&'
%left EQEQ NE
%left '<' '>' LE GE INSTANCEOF
%left LSHIFT RSHIFT LOGRSHIFT
%left '-' '+'
%left '*' '/' '%'
%left NEG POS NOT    /* unary minus,plus,not */

%%

Goal:
	  CompilationUnit
	;

EmptyCmds: | EmptyCmds ';'

CompilationUnit:
	  PackageDeclarationopt ImportDeclarations TypeDeclarations
	;

PackageDeclaration:
	  PACKAGE Name ';'
	;

ImportDeclarations:
	  ImportDeclaration EmptyCmds
	| ImportDeclarations ImportDeclaration EmptyCmds
	;

ImportDeclaration:
	  SingleTypeImportDeclaration
	| TypeImportOnDemandDeclaration
	;

SingleTypeImportDeclaration:
	  IMPORT Name ';'
	;

TypeImportOnDemandDeclaration:
	  IMPORT Name '.' '*' ';'
	;

TypeDeclarations:
	  TypeDeclaration EmptyCmds
	| TypeDeclarations TypeDeclaration EmptyCmds
	;

TypeDeclaration:
	  ClassDeclaration
	| InterfaceDeclaration
	;

ClassDeclaration:
	  Modifiersopt CLASS IDENTIFIER Superopt Interfacesopt ClassBody
	;

ClassBody:
	  '{' ClassBodyDeclarationsopt '}'
	;

ClassBodyDeclarations:
	  ClassBodyDeclaration EmptyCmds
	| ClassBodyDeclarations ClassBodyDeclaration EmptyCmds
	;

ClassBodyDeclaration:
	  ClassMemberDeclaration
	| StaticInitializer
	| ConstructorDeclaration
	;

ClassMemberDeclaration:
	  FieldDeclaration
	| MethodDeclaration
	;

MethodDeclaration:
	MethodHeader Block
	;

Block:
	  '{' BlockStatementsopt '}'
	;

BlockStatements:
	  BlockStatement EmptyCmds
	| BlockStatements BlockStatement EmptyCmds
	;

BlockStatement:
	  LocalVariableDeclaration ';'
	| Statement
	;

LocalVariableDeclaration:
	  Type VariableDeclarators
	;

VariableDeclarators:
	  VariableDeclarator
	| VariableDeclarators ',' VariableDeclarator
	;

VariableDeclarator:
	  VariableDeclaratorId
	| VariableDeclaratorId '=' VariableInitializer
	;

VariableDeclaratorId:
	  IDENTIFIER
	| VariableDeclaratorId '[' ']'
	;

VariableInitializer:
	  Expression
	| ArrayInitializer
	;

Statement:
	  Block
	| BreakStatement
	| ContinueStatement
	| DoStatement
	| ExpressionStatement
	| ForStatement
	| IfThenElseStatement
	| LabeledStatement
	| ReturnStatement
	| SwitchStatement
	| SynchronizedStatement
	| ThrowStatement
	| TryStatement
	| WhileStatement
	;

BreakStatement:
	  BREAK Identifieropt ';'
	;

ContinueStatement:
	  CONTINUE Identifieropt ';'
	;

DoStatement:
	  DO Statement WHILE '(' Expression ')' ';'
	;

ExpressionStatement:
	  StatementExpression ';'
	;

StatementExpression:
	  Assignment
	| PreIncrementExpression
	| PreDecrementExpression
	| PostIncrementExpression
	| PostDecrementExpression
	| MethodInvocation
	| ClassInstanceCreationExpression
	;

StatementExpressionList:
	  StatementExpression
	| StatementExpressionList ',' StatementExpression
	;

ForStatement:
	  FOR '(' ForInitopt ';' Expressionopt ';' ForUpdateopt ')' Statement
	;

ForInit:
	  StatementExpressionList
	| LocalVariableDeclaration
	;

ForUpdate:
	  StatementExpressionList
	;

IfThenElseStatement:
	  IF '(' Expression ')' Statement ELSE Statement
	| IF '(' Expression ')' Statement 
	;

LabeledStatement:
	  IDENTIFIER ':' Statement
	;

ReturnStatement:
	  RETURN Expressionopt ';'
	;

SwitchStatement:
	  SWITCH '(' Expression ')' SwitchBlock
	;

SwitchBlock:
	  '{' SwitchBlockStatementGroups SwitchLabelsopt '}'
	|  '{' SwitchLabelsopt '}'
	;

SwitchBlockStatementGroups:
	  SwitchBlockStatementGroup
	| SwitchBlockStatementGroups SwitchBlockStatementGroup
	;

SwitchBlockStatementGroup:
	  SwitchLabels BlockStatements
	;

SwitchLabels:
	  SwitchLabel
	| SwitchLabels SwitchLabel
	;

SwitchLabel:
	  CASE ConstantExpression ':'
	| DEFAULT ':'
	;

SynchronizedStatement:
	  SYNCHRONIZED '(' Expression ')' Block
	;

ThrowStatement:
	  THROW Expression ';'
	;

TryStatement:
	  TRY Block Catches
	| TRY Block Catchesopt Finally
	;

Catches:
	  CatchClause
	| Catches CatchClause
	;

CatchClause:
	  CATCH '(' FormalParameter ')' Block
	;

Finally:
	  FINALLY Block
	;

WhileStatement:
	  WHILE '(' Expression ')' Statement
	;

Assignment:
	  LeftHandSide AssignmentOperator AssignmentExpression
	;

AssignmentExpression:
	  ConditionalExpression
	| Assignment
	;

ConditionalExpression:
	  ConditionalOrExpression
	| ConditionalOrExpression '?' Expression ':' ConditionalExpression
	;

InterfaceDeclaration:
	  Modifiersopt INTERFACE IDENTIFIER ExtendsInterfacesopt InterfaceBody
	;

ExtendsInterfaces:
	  EXTENDS InterfaceType
	| ExtendsInterfaces ',' InterfaceType
	;

InterfaceBody:
	  '{' InterfaceMemberDeclarationsopt '}'
	;

InterfaceMemberDeclarations:
	  InterfaceMemberDeclaration EmptyCmds
	| InterfaceMemberDeclarations InterfaceMemberDeclaration EmptyCmds
	;

InterfaceMemberDeclaration:
	  ConstantDeclaration
	| AbstractMethodDeclaration
	;

ConstantDeclaration:
	  FieldDeclaration
	;

AbstractMethodDeclaration:
	  MethodHeader ';'
	;

Name:
	  SimpleName
	| QualifiedName
	;

SimpleName:
	  IDENTIFIER
	;

QualifiedName:
	  Name '.' IDENTIFIER
	;

Modifiers:
	  Modifier
	| Modifiers Modifier
	;

Modifier:
	  PUBLIC
	| PROTECTED
	| PRIVATE
	| STATIC
	| ABSTRACT
	| FINAL
	| NATIVE
	| SYNCHRONIZED
	| TRANSIENT
	| VOLATILE
	;

Super:
	  EXTENDS ClassType
	;

Interfaces:
	  IMPLEMENTS InterfaceTypeList
	;

InterfaceTypeList:
	  InterfaceType
	| InterfaceTypeList ',' InterfaceType
	;

FieldDeclaration:
	  Modifiersopt Type VariableDeclarators ';'
	;

MethodHeader:
	  Modifiersopt Type MethodDeclarator Throwsopt
	| Modifiersopt VOID MethodDeclarator Throwsopt
	;

MethodDeclarator:
	  IDENTIFIER '(' FormalParameterListopt ')'
	| MethodDeclarator '[' ']'
	;

FormalParameterList:
	  FormalParameter
	| FormalParameterList ',' FormalParameter
	;

FormalParameter:
	  Type VariableDeclaratorId
	;

Throws:
	  THROWS ClassTypeList
	;

ClassTypeList:
	  ClassType
	| ClassTypeList ',' ClassType
	;

StaticInitializer:
	  STATIC Block
	;

ConstructorDeclaration:
	  Modifiersopt ConstructorDeclarator Throwsopt ConstructorBody
	;

ConstructorDeclarator:
	  SimpleName '(' FormalParameterListopt ')'
	;

ConstructorBody:
	  '{' ExplicitConstructorInvocation BlockStatementsopt '}'
	| Block
	;

ExplicitConstructorInvocation:
	  THIS '(' ArgumentListopt ')' ';'
	| SUPER '(' ArgumentListopt ')' ';'
	;

ArrayInitializer:
	  '{' VariableInitializers '}'
	| '{' VariableInitializers ',' '}'
	;

VariableInitializers:
	  VariableInitializer
	| VariableInitializers ',' VariableInitializer
	;

Primary:
	  PrimaryNoNewArray
	| ArrayCreationExpression
	;

PrimaryNoNewArray:
	  Literal
	| THIS
	| '(' Expression ')'
	| ClassInstanceCreationExpression
	| FieldAccess
	| MethodInvocation
	| ArrayAccess
	;

ClassInstanceCreationExpression:
	  NEW ClassType '(' ArgumentListopt ')'
	;

ArgumentList:
	  Expression
	| ArgumentList ',' Expression
	;

ArrayCreationExpression:
	  NEW PrimitiveType DimExprs Dimsopt
	| NEW ClassOrInterfaceType DimExprs Dimsopt
	;

DimExprs:
	  DimExpr
	| DimExprs DimExpr
	;

DimExpr:
	  '[' Expression ']'
	;

Dims:
	  '[' ']'
	| Dims '[' ']'
	;

FieldAccess:
	  Primary '.' IDENTIFIER
	| SUPER '.' IDENTIFIER
	;

MethodInvocation:
	  Name '(' ArgumentListopt ')'
	| Primary '.' IDENTIFIER '(' ArgumentListopt ')'
	| SUPER '.' IDENTIFIER '(' ArgumentListopt ')'
	;

ArrayAccess:
	  Name '[' Expression ']'
	| PrimaryNoNewArray '[' Expression ']'
	;

PostfixExpression:
	  Primary
	| Name
	| PostIncrementExpression
	| PostDecrementExpression
	;

PostIncrementExpression:
	  PostfixExpression PLUSPLUS
	;

PostDecrementExpression:
	  PostfixExpression MINUSMINUS
	;

UnaryExpression:
	  PreIncrementExpression
	| PreDecrementExpression
	| '+' UnaryExpression
	| '-' UnaryExpression
	| UnaryExpressionNotPlusMinus
	;

PreIncrementExpression:
	  PLUSPLUS UnaryExpression
	;

PreDecrementExpression:
	  MINUSMINUS UnaryExpression
	;

UnaryExpressionNotPlusMinus:
	  PostfixExpression
	| '~' UnaryExpression
	| '!' UnaryExpression
	| CastExpression
	;

CastExpression:
	  '(' PrimitiveType Dimsopt ')' UnaryExpression
	| '(' Expression ')' UnaryExpressionNotPlusMinus
	| '(' Name Dims ')' UnaryExpressionNotPlusMinus
	;

RelationalExpression:
	  UnaryExpression
	| RelationalExpression '*' UnaryExpression
	| RelationalExpression '/' UnaryExpression
	| RelationalExpression '%' UnaryExpression
	| RelationalExpression '+' RelationalExpression
	| RelationalExpression '-' RelationalExpression
	| RelationalExpression LSHIFT RelationalExpression
	| RelationalExpression RSHIFT RelationalExpression
	| RelationalExpression LOGRSHIFT RelationalExpression
	| RelationalExpression '<' RelationalExpression
	| RelationalExpression '>' RelationalExpression
	| RelationalExpression LE RelationalExpression
	| RelationalExpression GE RelationalExpression
	| RelationalExpression INSTANCEOF ReferenceType
	;

EqualityExpression:
	  RelationalExpression
	| EqualityExpression EQEQ RelationalExpression
	| EqualityExpression NE RelationalExpression
	;

AndExpression:
	  EqualityExpression
	| AndExpression '&' EqualityExpression
	;

ExclusiveOrExpression:
	  AndExpression
	| ExclusiveOrExpression '^' AndExpression
	;

InclusiveOrExpression:
	  ExclusiveOrExpression
	| InclusiveOrExpression '|' ExclusiveOrExpression
	;

ConditionalAndExpression:
	  InclusiveOrExpression
	| ConditionalAndExpression ANDAND InclusiveOrExpression
	;

ConditionalOrExpression:
	  ConditionalAndExpression
	| ConditionalOrExpression OROR ConditionalAndExpression
	;

LeftHandSide:
	  Name
	| FieldAccess
	| ArrayAccess
	;

AssignmentOperator:
	  '='
	| EQMUL
	| EQDIV
	| EQMOD
	| EQADD
	| EQSUB
	| EQLSHIFT
	| EQRSHIFT
	| EQLOGRSHIFT
	| EQAND
	| EQXOR
	| EQOR
	;

Expression:
	  AssignmentExpression
	;

ConstantExpression:
	  Expression
	;	

Type:
	  PrimitiveType
	| ReferenceType
	;

PrimitiveType:
	  NumericType
	| BOOLEAN
	;

NumericType:
	  IntegralType
	| FloatingPointType
	;

IntegralType:
	  BYTE
	| SHORT
	| INT
	| LONG
	| CHAR
	;

FloatingPointType:
	  FLOAT
	| DOUBLE
	;

ReferenceType:
	  ClassOrInterfaceType
	| ArrayType
	;

ClassOrInterfaceType:
	  Name
	;

ClassType:
	  ClassOrInterfaceType
	;

InterfaceType:
	  ClassOrInterfaceType
	;

ArrayType:
	  PrimitiveType '[' ']'
	| Name '[' ']'
	| ArrayType '[' ']'

	;

Literal:
	  INTEGER_LITERAL
	| FLOAT_LITERAL
	| BOOLEAN_LITERAL
	| CHARACTER_LITERAL
	| STRING_LITERAL
	| NULL
	;

ArgumentListopt: /*empty*/ | ArgumentList ;
BlockStatementsopt: /*empty*/ | BlockStatements ;
Catchesopt: /*empty*/ | Catches ;
ClassBodyDeclarationsopt: /*empty*/ | ClassBodyDeclarations ;
Dimsopt: /*empty*/ | Dims ;
Expressionopt: /*empty*/ | Expression ;
ExtendsInterfacesopt: /*empty*/ | ExtendsInterfaces ;
ForInitopt: /*empty*/ | ForInit ;
ForUpdateopt: /*empty*/ | ForUpdate ;
FormalParameterListopt: /*empty*/ | FormalParameterList ;
Identifieropt: /*empty*/ | IDENTIFIER ;
InterfaceMemberDeclarationsopt: /*empty*/ | InterfaceMemberDeclarations ;
Interfacesopt: /*empty*/ | Interfaces ;
Modifiersopt: /*empty*/ | Modifiers ;
PackageDeclarationopt: /*empty*/ | PackageDeclaration ;
Superopt: /*empty*/ | Super ;
SwitchLabelsopt: /*empty*/ | SwitchLabels ;
Throwsopt: /*empty*/ | Throws ;

%%

/* Override yyerror */
public
void
yyverror(String msg) throws ParseException { yyerror(msg,true,true); }

